/* -*- ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the iKtai Library.

   The iKtai Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The iKtai Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the iKtai Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoBaseTestRunner.m,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#include "CocoBaseTestRunner.h"
#include <CocoUnit/CocoTestSuite.h>
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestFailure.h>
#include <sys/time.h>
#import <objc/objc-runtime.h>

enum
{
  COCOUNIT_RUNLISTENER_STATUS_ERROR = 1,
  COCOUNIT_RUNLISTENER_STATUS_FAILURE = 2,
};

NSString* const COCOUNIT_SUITE_METHODNAME = @"suite";
//static int fgMaxMessageLength = 500;
//static bool fgFilterStack = true;

@implementation CocoBaseTestRunner

+(id) testRunner
{
  return [ self listener ];
}

-(void) startTest:(CocoTest*) test
{
  [ self testStarted: [ test description ] ];
}

-(void) endTest:(CocoTest*) test
{
  [ self testEnded: [ test description ] ];
}

-(void) addError:(NSException*) error test:(CocoTest*) test
{
  [ self testFailed: COCOUNIT_RUNLISTENER_STATUS_ERROR
	 test: test exception: error ];
}

-(void) addFailure:(CocoAssertionFailedError*) failure test:(CocoTest*) test
{
  [ self testFailed: COCOUNIT_RUNLISTENER_STATUS_FAILURE
	 test: test exception:(NSException*) failure ];
}

/*
 * TestRunListener implementation
 */
-(void) testStarted:(NSString*) testName 
{
}

-(void) testEnded:(NSString*) testName
{
}

-(void) testFailed:(int) status test:(CocoTest*) test exception:(NSException*) e
{
}

-(CocoTest*) testFromString:(NSString*) suiteClassName
{
  if ([ suiteClassName length ] <= 0)
    {
      [ self clearStatus ];
      return nil;
    }
  Class testClass = nil;
  @try
  {
    testClass = NSClassFromString (suiteClassName);
    if (!testClass)
      {
	[ self runFailed: [ NSString stringWithFormat: @"Class not found %@",
				     suiteClassName ] ];
	return nil;
      }
  }
  @catch (NSException e)
  {
    [ self runFailed: [ NSString stringWithFormat: @"Error: %@", e ] ];
    return nil;
  }
  Method suiteMethod = nil;
  SEL s = NSSelectorFromString (COCOUNIT_SUITE_METHODNAME);
  suiteMethod = class_getClassMethod (testClass, s);
  if (!suiteMethod)
    {
      suiteMethod = class_getInstanceMethod (testClass, s);
      if (suiteMethod)
	{
	  [ self runFailed: @"suite() method must be class method" ];
	  return nil;
	}
      // try to extract a test suite automatically
      [ self clearStatus ];
      return [ CocoTestSuite testSuiteWithClass: testClass ];
    }
  CocoTest* test = nil;
  id cid = objc_getClass ([ suiteClassName cString ]);
  test = objc_msgSend (cid, s);
  if (!test)
    return nil;
  [ self clearStatus ];
  return test;
}

-(NSString*) elapsedTimeAsString:(struct timeval*) runTime
{
  return [ NSString stringWithFormat: @"%ld.%06ld",
		    (long)runTime->tv_sec, (long)runTime->tv_usec ];
}

-(void) runFailed:(NSString*) message
{
}

-(void) clearStatus
{
  // Belongs in the GUI TestRunner class
}

@end
