/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: ArrayTest.m,v 1.7 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 *
 */

#include <CocoUnit/CocoTextUiTestRunner.h>
#include <CocoUnit/CocoTestResult.h>
#include "ArrayTest.h"

@implementation ArrayTest

-(void) setUp
{
  fEmpty = [ [ NSMutableArray alloc ] init ];
  fFull = [ [ NSMutableArray alloc ] init ];
  int i;
  for (i = 1; i <= 3; i ++)
    [ fFull addObject: [ NSNumber numberWithInt: i ] ];
}

-(void) tearDown
{
  [ fEmpty release ];
  [ fFull release ];
}

-(void) testCapacity
{
  int size = [ fFull count ];
  int i;
  for (i = 0; i < 100; i ++)
    [ fFull addObject: [ NSNumber numberWithInt: i ] ];
  [ self assertTrue: [ fFull count ] == 100 + size ];
}

-(void) testClone
{
  NSMutableArray* clone = [ [ fFull copy ] autorelease ];
  [ self assertTrue: [ clone count ] == [ fFull count ] ];
  [ self assertTrue: [ clone containsObject: [ NSNumber numberWithInt: 1 ] ] ];
}

-(void) testContains
{
  [ self assertTrue: [ fFull containsObject: [ NSNumber numberWithInt: 1 ] ] ];  
  [ self assertFalse: [ fEmpty containsObject: [ NSNumber numberWithInt: 1 ] ] ];
}

-(void) testElementAt
{
  NSNumber* i = (NSNumber*)[ fFull objectAtIndex: 0 ];
  [ self assertTrue: [ i intValue ] == 1 ];
  @try
  { 
    [ fFull objectAtIndex: [ fFull count ] ];
  }
  @catch (NSException* e)
  {
    if ([ [ e name ] isEqual: NSRangeException ])
      return;
  }
  [ self fail: @"Should raise an NSRangeException" ];
}

-(void) testRemoveAll
{
  [ fFull removeAllObjects ];
  [ fEmpty removeAllObjects ];
  [ self assertTrue: [ fFull count ] == 0 ];
  [ self assertTrue: [ fEmpty count ] == 0 ]; 
}

-(void) testRemoveElement
{
  [ fFull removeObject: [ NSNumber numberWithInt: 3 ] ];
  [ self assertFalse: [ fFull containsObject: [ NSNumber numberWithInt: 3 ] ] ];
}

@end

