/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: MoneyBag.m,v 1.2 2003/11/24 03:34:30 tsutsumi Exp $
 */

/** @file
 *
 */

#include "MoneyBag.h"
#include "Money.h"

@implementation MoneyBag

-(id) init
{
  monies = [ [ NSMutableArray arrayWithCapacity: 5 ] retain ];
  return [ self autorelease ];
}

-(void)dealloc
{
  [ monies release ];
  [ super dealloc ];
}

+(id) moneyBagWithMonies:(Money**) bag count:(unsigned) count
{
  MoneyBag* newBag = [ MoneyBag new ];
  unsigned i;
  for (i = 0; i < count; i ++)
    [ newBag appendMoney: bag[i] ];
  return newBag;
}

+(id) moneyBagWithMoney:(Money*) m1 money:(Money*) m2
{
  MoneyBag* newBag = [ MoneyBag new ];
  [ newBag appendMoney: m1 ];
  [ newBag appendMoney: m2 ];
  return newBag;
}

+(id) moneyBagWithMoney:(Money*) m bag:(MoneyBag*) bag
{
  MoneyBag* newBag = [ MoneyBag new ];
  [ newBag appendMoney: m ];
  [ newBag appendMoneyBag: bag ];
  return newBag;
}

+(id) moneyBagWithMoneyBag:(MoneyBag*) m1 bag:(MoneyBag*) m2
{
  MoneyBag* newBag = [ MoneyBag new ];
  [ newBag appendMoneyBag: m1 ];
  [ newBag appendMoneyBag: m2 ];
  return newBag;
}

-(IMoney*) add:(IMoney*) m
{
  return [ m addMoneyBag: self ];
}

-(IMoney*) simplify
{
  if ([ monies count ] == 1)
    return [ [ monies objectEnumerator ] nextObject ];
  return self;
}

-(IMoney*) addMoney:(Money*) m
{
  return [ [ MoneyBag moneyBagWithMoney: m bag: self ] simplify ];
}

-(IMoney*) addMoneyBag:(MoneyBag*) s
{
  return [ [ MoneyBag moneyBagWithMoneyBag: s bag: self ] simplify ];
}

-(void) appendMoneyBag:(MoneyBag*) aBag
{
  Money* m;
  NSEnumerator* e = [ aBag->monies objectEnumerator ];
  while ((m = [ e nextObject ]) != nil)
    [ self appendMoney: m ];
}

-(Money*) find:(NSString*) currency
{
  NSEnumerator* e = [ monies objectEnumerator ];
  Money* m;
  while ((m = [ e nextObject ]) != nil)
    {
      if ([ [ m currency ] isEqual: currency ])
	return m;
    }
  return nil;
}

-(void) appendMoney:(Money*) aMoney
{
  IMoney* old = [ self find: [ aMoney currency ] ];
  if (!old)
    [ monies addObject: aMoney ];
  else
    {
      [ monies removeObject: old ];
      IMoney* sum = [ old add: aMoney ];
      if (![ sum isNil ])
	[ monies addObject: sum ];
    }
}

-(bool) contains:(Money*) aMoney
{
  Money* m = [ self find: [ aMoney currency ] ];
  return [ m amount ] == [ aMoney amount ];
}

-(bool) isEqual:(NSObject*) anObject
{
  if ([ self isNil ])
    if ([ anObject isKindOfClass: [ IMoney class ] ])
      return [ (IMoney*)anObject isNil ];
  if ([ anObject isKindOfClass: [ MoneyBag class ] ])
    {
      MoneyBag* aBag = (MoneyBag*)anObject;
      if ([ aBag->monies count ] != [ monies count ])
	return false;
      NSEnumerator* e = [ monies objectEnumerator ];
      Money* m;
      while (( m = [ e nextObject ]) != nil)
	{
	  if (![ aBag contains: m ])
	    return false;
	}
      return true;
    }
  return false;
}

-(unsigned) hash
{
  unsigned hash = 0;
  NSEnumerator* e = [ monies objectEnumerator ];
  IMoney* m;
  while ((m = [ e nextObject ]) != nil)
    {
      hash ^= [ m hash ];
    }
  return hash;
}

-(bool) isNil
{
  return [ monies count ] == 0;
}

-(IMoney*) multiply:(int) factor
{
  MoneyBag* result = [ MoneyBag new ];
  if (factor != 0)
    {
      NSEnumerator* e = [ monies objectEnumerator ];
      Money* m;
      while ((m = [ e nextObject ]) != nil)
	{
	  [ result appendMoney:(Money*) [ m multiply: factor ] ];
	}
    }
  return result;
}

-(IMoney*) negate
{
  MoneyBag* result = [ MoneyBag new ];
  NSEnumerator* e = [ monies objectEnumerator ];
  Money* m;
  while ((m = [ e nextObject ]) != nil)
    {
      [ result appendMoney:(Money*) [ m negate ] ];
    }
  return result;
}

-(IMoney*) subtract:(IMoney*) m
{
  return [ self add: [ m negate ] ];
}

-(NSString*) description
{
  NSMutableString* buffer;
  buffer = [ [ [ NSMutableString alloc ] init ] autorelease ];
  [ buffer appendString: @"{" ];
  NSEnumerator* e = [ monies objectEnumerator ];
  Money* m;
  while ((m = [ e nextObject ]) != nil)
    {
      [ buffer appendString: [ m description ] ];
    }
  [ buffer appendString: @"}" ];
  return buffer;
}

@end

