/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: ComparisonFailureTest.m,v 1.4 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 *
 */

#include "ComparisonFailureTest.h"
#include <CocoUnit/CocoComparisonFailure.h>
#include <CocoUnit/CocoTestCase.h>

@implementation ComparisonFailureTest

-(void) testComparisonErrorMessage
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: @"a" expected: @"b" actual: @"c" ];
  [ self assertEquals: @"a expected:<b> but was:<c>" actual: [ failure reason ] ];
}

-(void) testComparisonErrorStartSame
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: nil expected: @"ba" actual: @"bc" ];
  [ self assertEquals: @"expected:<..?a> but was:<..?c>" actual: [ failure reason ]];
}

-(void) testComparisonErrorEndSame
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: nil expected: @"ab" actual: @"cb" ];
  [ self assertEquals: @"expected:<a?..> but was:<c?..>" actual: [ failure reason ]];
}

-(void) testComparisonErrorSame
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: nil expected: @"ab" actual: @"ab" ];
  [ self assertEquals: @"expected:<ab> but was:<ab>" actual: [ failure reason ]];
}

-(void) testComparisonErrorStartAndEndSame
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: nil expected: @"abc" actual: @"adc" ];
  [ self assertEquals: @"expected:<..?b?..> but was:<..?d?..>"
	 actual: [ failure reason ] ];
}

-(void) testComparisonErrorStartSameComplete
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: nil expected: @"ab" actual: @"abc" ];
  [ self assertEquals: @"expected:<..?> but was:<..?c>" actual: [ failure reason ] ];
}

-(void) testComparisonErrorEndSameComplete
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: nil expected: @"bc" actual: @"abc" ];
  [ self assertEquals: @"expected:<?..> but was:<a?..>" actual: [ failure reason ] ];
}

-(void) testComparisonErrorOverlapingMatches
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure
	      comparisonFailure: nil expected: @"abc" actual: @"abbc"];
  [ self assertEquals: @"expected:<..??..> but was:<..?b?..>"
	 actual: [ failure reason ] ];
}

-(void) testComparisonErrorOverlapingMatches2
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure comparisonFailure: nil expected: @"abcdde"
				    actual: @"abcde" ];
  [ self assertEquals: @"expected:<..?d?..> but was:<..??..>"
	 actual: [ failure reason ] ];
}

-(void) testComparisonErrorWithActualNull
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure comparisonFailure: nil
				    expected: @"a" actual: nil ];
  [ self assertEquals: @"expected:<a> but was:<<null>>" actual: [ failure reason ] ];
}

-(void) testComparisonErrorWithExpectedNull
{
  CocoComparisonFailure* failure;
  failure = [ CocoComparisonFailure comparisonFailure: nil
				    expected: nil actual: @"a" ];
  [ self assertEquals: @"expected:<<null>> but was:<a>" actual: [ failure reason ] ];
}
@end
// Local variables: ***
// compile-command: "xcodebuild -project ../../CocoUnit.xcode -target CocoUnitTest" ***
// End: ***
