/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: DoublePrecisionAssertTest.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 *
 */

#include "DoublePrecisionAssertTest.h"

#include <CocoUnit/CocoAssertionFailedError.h>
#include <CocoUnit/CocoTestCase.h>
#include <CocoUnit/CocoAssert.h>
#include <math.h>

@implementation DoublePrecisionAssertTest

/**
 * Test for the special Double.NaN value.
 */
-(void) testAssertEqualsNaNFails
{
  @try
  {
    [ self assertEquals: 1.234 actual: NAN delta: 0.0 ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertNaNEqualsFails
{
  @try
  {
    [ self assertEquals: NAN actual: 1.234 delta: 0.0 ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertNaNEqualsNaNFails
{
  @try
  {
    [ self assertEquals: NAN actual: NAN delta: 0.0 ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertPosInfinityNotEqualsNegInfinity
{
  @try
  {
    [ self assertEquals: INFINITY actual: -INFINITY delta: 0.0 ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertPosInfinityNotEquals
{
  @try
  {
    [ self assertEquals: INFINITY actual: 1.23 delta: 0.0 ];
  }
  @catch (CocoAssertionFailedError* e)
  {
    return;
  }
  [ self fail ];
}

-(void) testAssertPosInfinityEqualsInfinity
{
  [ self assertEquals: INFINITY actual: INFINITY delta: 0.0 ];
}

-(void) testAssertNegInfinityEqualsInfinity
{
  [ self assertEquals: -INFINITY actual: -INFINITY delta: 0.0 ];
}
@end
// Local variables: ***
// compile-command: "xcodebuild -project ../../CocoUnit.xcode -target CocoUnitTest" ***
// End: ***
