/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Framework.

   The CocoUnit Framework is free software; you can redistribute
   it and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTextUiResultPrinter.h,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 *
 */

#ifndef COCOTEXTUIRESULTPRINTER_H
#define COCOTEXTUIRESULTPRINTER_H

#include <CocoUnit/CocoTestListener.h>
#include <stdio.h>
#include <sys/time.h>
#include <stdbool.h>

@class CocoWriter;
@class CocoTestResult;
@class CocoTestFailure;

/**
 *
 */
@interface CocoTextUiResultPrinter : CocoTestListener {
  CocoWriter *writer;
  int column;
  bool verbose;
  int runTests;
}

+(id) resultPrinter:(CocoWriter*) resultWriter;

-(id) init;

-(id) initWithWriter:(CocoWriter*) resultWriter;

-(void) print:(CocoTestResult*) result runTime:(struct timeval*) runTime;

-(void) printHeader:(struct timeval*) runTime;

-(void) printErrors:(CocoTestResult*) result;

-(void) printFailures:(CocoTestResult*) result;

-(void) printDefects:(NSEnumerator*) booBoos count:(int) count type:(NSString*) type;

-(void) printDefect:(CocoTestFailure*) booBoo count:(int) count;

-(void) printDefectHeader:(CocoTestFailure*) booBoo count:(int) count;

-(void) printFooter:(CocoTestResult*) result;

/**
 * Returns the formatted string of the elapsed time.
 */
-(NSString*) elapsedTimeAsString:(struct timeval*) runTime;

-(CocoWriter*) writer;

/**
 * @see CocoTestListener#addTest:error:
 */
-(void) addTest:(CocoTest*) test error:(NSException*) t;

/**
 * @see CocoTestListener#addTest:failure:
 */
-(void) addTest:(CocoTest*) test failure:(CocoAssertionFailedError*) failure;

/**
 * @see CocoTestListener#endTest:
 */
-(void) endTest:(CocoTest*) test;

/**
 * @see CocoTestListener#startTest:
 */
-(void) startTest:(CocoTest*) test;

-(void) verbose: (bool) isEnable;

@end

#endif // COCOTEXTUIRESULTPRINTER_H
