/* -*- ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the iKtai Library.

   The iKtai Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The iKtai Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the iKtai Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTextUiTestRunner.m,v 1.3 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#include "CocoTextUiTestRunner.h"
#include <CocoUnit/CocoTestSuite.h>
#include <CocoUnit/CocoTestResult.h>
#include <CocoUnit/CocoTestFailure.h>
#include <CocoUnit/CocoWriter.h>
#include <CocoUnit/CocoStreamWriter.h>
#include <CocoUnit/CocoTextUiResultPrinter.h>
#include <sys/time.h>
#import <objc/objc-runtime.h>

@implementation CocoTextUiTestRunner

+(id) testRunner
{
  return [ self testRunnerWithStream: stdout ];
}

+(id) testRunnerWithStream:(FILE*) stream
{
  id aWriter = [ [ CocoStreamWriter alloc ] initWithStream: stream ];
  id aRunner = [ self testRunnerWithWriter: aWriter ];
  [ aWriter release ];
  return aRunner;
}

+(id) testRunnerWithWriter:(CocoWriter*) aWriter
{
  id aPrinter = [ [ CocoTextUiResultPrinter alloc ] initWithWriter: aWriter ];
  id aRunner = [ self testRunnerWithPrinter: aPrinter ];
  [ aPrinter release ];
  return aRunner;
}

+(id) testRunnerWithPrinter:(CocoTextUiResultPrinter*) aPrinter
{
  return [ [ [ self alloc ] initWithPrinter: aPrinter ] autorelease ];
}

+(void) run:(Class) testClass
{
  [ self runWithSuite: [ CocoTestSuite testSuiteWithClass: testClass ] ];
}

+(CocoTestResult*) runWithSuite:(CocoTest*) suite
{
  return [ [ self testRunner ] doRun: suite ];
}

-(id) init
{
  return [ self initWithStream: stdout ];
}

-(id) initWithStream:(FILE*) stream
{
  id aWriter = [ [ CocoStreamWriter alloc ] initWithStream: stream ];
  id aRunner = [ self initWithWriter: aWriter ];
  [ aWriter release ];
  return aRunner;
}

-(id) initWithWriter:(CocoWriter*) aWriter
{
  id aPrinter = [ [ CocoTextUiResultPrinter alloc ] initWithWriter: aWriter];
  id aRunner = [ self initWithPrinter: aPrinter ];
  [ aPrinter release ];
  return aRunner;
}

-(id) initWithPrinter:(CocoTextUiResultPrinter*) aPrinter
{
  [ super init ];
  printer = [ aPrinter retain ];
  return self;
}

-(void) dealloc
{
  [ printer release ];
  [ super dealloc ];
}

/**
 * Creates the TestResult to be used for the test run.
 */
-(CocoTestResult*) createTestResult
{
  return [ CocoTestResult testResult ];
}

-(CocoTestResult*) doRun:(CocoTest*) suite
{
  struct timeval startTime;
  struct timeval endTime;
  struct timeval runTime;
  CocoTestResult* result = [ self createTestResult ];
  [ printer verbose: verbose ];
  [ result addListener: printer ];
  gettimeofday (&startTime, NULL);
  [ suite runWithResult: result ];
  gettimeofday (&endTime, NULL);
  timersub(&endTime, &startTime, &runTime);
  [ printer print: result runTime: &runTime ];
  return result;
}

-(void) runFailed:(NSString*) message
{
  fprintf (stderr, "%s\n", [ message cString ]);
  exit (FAILURE_EXIT);
}

-(void) printer:(CocoTextUiResultPrinter*) resultPrinter
{
  printer = [ resultPrinter retain ];
}

-(void) verbose:(bool) isEnable
{
  verbose = isEnable;
}

@end
